//
//  Assignment.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "Assignment.h"
#import "TaxiAPIClient.h"

@implementation Assignment

- (id)initWithAttributes:(NSDictionary *)attributes
{
    self = [self init];
    
    if (self){
        self.status = [[self class] statusFromString:attributes[@"status"]];
        [self setValue:[[Ride alloc] initWithAttributes:attributes[@"ride"]] forKey:@"ride"];
    }
    
    return self;
}

// "Traducción" de los estados de string a enum
+ (NSDictionary *)statusDisplayNames
{
    return @{@(TaxiAssignmentStatusPending) : @"pending",
             @(TaxiAssignmentStatusAccepted) : @"accepted",
             @(TaxiAssignmentStatusRejected) : @"rejected"};
}

// De enum a string
- (NSString *)statusDisplayName
{
    return [[self class] statusDisplayNames][@(self.status)];
}

// De string a enum
+ (TaxiAssignmentStatus)statusFromString:(NSString *)status
{
    return [[[[[self class] statusDisplayNames] allKeysForObject:status] lastObject] integerValue];
}

#pragma mark - Interfaz con API

// Devuelve la asignación actual
+ (void)currentAssignmentWithSuccess:(void (^)(Assignment *assignment))success failure:(void (^)(NSError *error))failure;
{
    [[TaxiAPIClient sharedClient] getAssignmentInformationWithSuccess:^(id JSON){
        Assignment *assignment = nil;
        if ([JSON count] > 0) {
            assignment = [[Assignment alloc] initWithAttributes:JSON];
        }
        if (success) {
            success(assignment);
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Acepta esta asignación
- (void)acceptAssignmentWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] acceptRideWithSuccess:^(id JSON) {
        self.status = TaxiAssignmentStatusAccepted;
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Rechaza esta asignación
- (void)rejectAssignmentWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] rejectRideWithSuccess:^(id JSON) {
        self.status = TaxiAssignmentStatusRejected;
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

@end
