//
//  CustomerAppDelegate.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "CustomerAppDelegate.h"
#import <AudioToolbox/AudioToolbox.h>
#import <AFNetworking.h>
#import <LUKeychainAccess.h>
#import <FUIAlertView.h>
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>

@implementation CustomerAppDelegate


- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Registrarse para notificaciones push
    [[UIApplication sharedApplication] registerForRemoteNotificationTypes:(UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound)];
    
    // Limpiar notificaciones pendientes
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
    
    // Configurar el indicador de actividad de AFNetworking
    [AFNetworkActivityIndicatorManager sharedManager].enabled = YES;
    
    return YES;
}


#pragma mark - Notificaciones Push

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    // Guardar token de dispositivo
    [[LUKeychainAccess standardKeychainAccess] setString:[deviceToken description] forKey:@"device"];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    NSLog(@"Error: %@", error);
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
    if (application.applicationState == UIApplicationStateActive) {
        // 1007 es el sonido por defecto en iOS al recibir SMS (tritono)
        AudioServicesPlaySystemSound(1007);

        FUIAlertView *alertView = [[FUIAlertView alloc] initWithTitle:NSLocalizedString(@"Aviso", @"Título aviso AlertView")
                                                              message:NSLocalizedString(userInfo[@"aps"][@"alert"][@"loc-key"], nil)
                                                             delegate:nil cancelButtonTitle:NSLocalizedString(@"Cerrar", @"Botón Cerrar AlertView")
                                                    otherButtonTitles:nil];
        alertView.titleLabel.textColor = [UIColor cloudsColor];
        alertView.titleLabel.font = [UIFont boldFlatFontOfSize:16];
        alertView.messageLabel.textColor = [UIColor cloudsColor];
        alertView.messageLabel.font = [UIFont flatFontOfSize:14];
        alertView.backgroundOverlay.backgroundColor = [[UIColor cloudsColor] colorWithAlphaComponent:0.5];
        alertView.alertContainer.backgroundColor = [UIColor midnightBlueColor];
        alertView.defaultButtonColor = [UIColor cloudsColor];
        alertView.defaultButtonShadowColor = [UIColor asbestosColor];
        alertView.defaultButtonFont = [UIFont boldFlatFontOfSize:16];
        alertView.defaultButtonTitleColor = [UIColor asbestosColor];
        [alertView show];
        
    }
    // Si no estaba activa, viene de cualquier otro estado.... normalmente inactivo
    // UIApplicationStateBackground o UIApplicationStateInactive

    // Limpiar notificaciones
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
    
    // Recargar datos
    [self.window.rootViewController performSelector:@selector(refreshRide) withObject:nil afterDelay:0];
}

@end
