//
//  CustomerMainViewController.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "CustomerMainViewController.h"
#import <MapKit/MapKit.h>
#import <LUKeychainAccess.h>
#import <FUIButton.h>
#import <UIBarButtonItem+FlatUI.h>
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>
#import <UINavigationBar+FlatUI.h>
#import "Taxi.h"
#import "Ride.h"
#import "Request.h"
#import "TaxiLocationDelegate.h"
#import "TaxiSegmentedControl.h"
#import "StarRatingView.h"

@interface CustomerMainViewController () <MKMapViewDelegate>

@property (nonatomic, assign) BOOL userTrackingEnabled;
@property (nonatomic, assign) BOOL isShowingAvailability;
@property (nonatomic, assign) BOOL isShowingCurrentRide;
@property (strong, nonatomic) NSTimer *availabilityTimer;

@property (strong, nonatomic) NSTimer *activeRideTimer;
@property (strong, nonatomic) Taxi *assignedTaxi;
@property (strong, nonatomic) Ride *activeRide;
@property (weak, nonatomic) IBOutlet UIView *activeRideView;
@property (weak, nonatomic) IBOutlet UILabel *activeRideStatusLabel;
@property (weak, nonatomic) IBOutlet UILabel *activeRideDistanceLabel;


@property (strong, nonatomic) Request *customerRequest;
@property (weak, nonatomic) IBOutlet UILabel *rideTimeLabel;
@property (weak, nonatomic) IBOutlet TaxiSegmentedControl *rideTimeControl;

@property (strong, nonatomic) Ride *unratedRide;
@property (weak, nonatomic) IBOutlet FUIButton *rateButton;
- (IBAction)rateLastRide;
@property (weak, nonatomic) IBOutlet UIView *ratePaneView;
@property (strong, nonatomic) StarRatingView *rateControl;
@property (weak, nonatomic) IBOutlet FUIButton *confirmRateButton;
@property (weak, nonatomic) IBOutlet FUIButton *cancelRateButton;
- (IBAction)confirmRate;
- (IBAction)cancelRate;

@property (weak, nonatomic) IBOutlet UIView *requestRideView;
@property (weak, nonatomic) IBOutlet FUIButton *requestRideButton;
@property (weak, nonatomic) IBOutlet FUIButton *cancelRideButton;
- (IBAction)cancelRequest;
- (IBAction)confirmRequest;


@property (weak, nonatomic) IBOutlet FUIButton *trackingButton;
@property (weak, nonatomic) IBOutlet UILabel *titleLabel;
@property (weak, nonatomic) IBOutlet MKMapView *mapView;
@property (weak, nonatomic) IBOutlet FUIButton *availabilityButton;
@property (weak, nonatomic) IBOutlet FUIButton *hailButton;
- (IBAction)hailTaxi;
- (IBAction)toggleAvailability;
- (IBAction)toggleTracking;
@end

@implementation CustomerMainViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    // Configurar Flat UI
    self.titleLabel.font = [UIFont boldFlatFontOfSize:18];
    self.titleLabel.textColor = [UIColor midnightBlueColor];
    self.titleLabel.shadowColor = [UIColor whiteColor];
    self.titleLabel.shadowOffset = CGSizeMake(0,1);

    // Vista trayecto en curso
    self.activeRideView.backgroundColor = [UIColor midnightBlueColor];
    self.activeRideStatusLabel.font = [UIFont boldFlatFontOfSize:18];
    self.activeRideStatusLabel.textColor = [UIColor cloudsColor];
    self.activeRideDistanceLabel.font = [UIFont boldFlatFontOfSize:18];
    self.activeRideDistanceLabel.textColor = [UIColor silverColor];

    // Vista confirmar petición
    self.requestRideView.backgroundColor = [UIColor midnightBlueColor];

    self.rideTimeLabel.font = [UIFont boldFlatFontOfSize:18];
    self.rideTimeLabel.textColor = [UIColor cloudsColor];

    self.rideTimeControl.selectedFont = [UIFont boldFlatFontOfSize:16];
    self.rideTimeControl.selectedFontColor = [UIColor cloudsColor];
    self.rideTimeControl.deselectedFont = [UIFont flatFontOfSize:16];
    self.rideTimeControl.deselectedFontColor = [UIColor cloudsColor];
    self.rideTimeControl.selectedColor = [UIColor amethystColor];
    self.rideTimeControl.deselectedColor = [UIColor asbestosColor];
    self.rideTimeControl.dividerColor = [UIColor midnightBlueColor];
    self.rideTimeControl.cornerRadius = 5.0;
    [self.rideTimeControl setTitle:NSLocalizedString(@"Ahora", @"Etiqueta control segmentado reserva - ahora") forSegmentAtIndex:0];
    [self.rideTimeControl setTitle:NSLocalizedString(@"En 1 h", @"Etiqueta control segmentado reserva - en 1 h") forSegmentAtIndex:1];
    [self.rideTimeControl setTitle:NSLocalizedString(@"En 2 h", @"Etiqueta control segmentado reserva - en 2 h") forSegmentAtIndex:2];

    self.requestRideButton.buttonColor = [UIColor emerlandColor];
    self.requestRideButton.shadowColor = [UIColor nephritisColor];
    self.requestRideButton.shadowHeight = 3.0f;
    self.requestRideButton.cornerRadius = 6.0f;
    self.requestRideButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.requestRideButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.requestRideButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];
    
    self.cancelRideButton.buttonColor = [UIColor alizarinColor];
    self.cancelRideButton.shadowColor = [UIColor pomegranateColor];
    self.cancelRideButton.shadowHeight = 3.0f;
    self.cancelRideButton.cornerRadius = 6.0f;
    self.cancelRideButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.cancelRideButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.cancelRideButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];
    
    // Vista puntuar
    self.ratePaneView.backgroundColor = [UIColor midnightBlueColor];
    
    self.rateControl = [[StarRatingView alloc]initWithFrame:CGRectMake(20, 20, 280, 50) andRating:0 withLabel:NO animated:YES];
    [self.ratePaneView addSubview:self.rateControl];

    self.confirmRateButton.buttonColor = [UIColor emerlandColor];
    self.confirmRateButton.shadowColor = [UIColor nephritisColor];
    self.confirmRateButton.shadowHeight = 3.0f;
    self.confirmRateButton.cornerRadius = 6.0f;
    self.confirmRateButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.confirmRateButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.confirmRateButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];
    
    self.cancelRateButton.buttonColor = [UIColor alizarinColor];
    self.cancelRateButton.shadowColor = [UIColor pomegranateColor];
    self.cancelRateButton.shadowHeight = 3.0f;
    self.cancelRateButton.cornerRadius = 6.0f;
    self.cancelRateButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.cancelRateButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.cancelRateButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];

    // Botones
    self.hailButton.buttonColor = [UIColor alizarinColor];
    self.hailButton.shadowColor = [UIColor pomegranateColor];
    self.hailButton.shadowHeight = 3.0f;
    self.hailButton.cornerRadius = 6.0f;
    self.hailButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.hailButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.hailButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];

    self.availabilityButton.buttonColor = [UIColor cloudsColor];
    self.availabilityButton.shadowColor = [UIColor silverColor];
    self.availabilityButton.shadowHeight = 2.0f;
    self.availabilityButton.cornerRadius = 3.0f;
    self.availabilityButton.titleLabel.font = [UIFont boldFlatFontOfSize:13];
    [self.availabilityButton setTitleColor:[UIColor asbestosColor] forState:UIControlStateNormal];
    [self.availabilityButton setTitleColor:[UIColor whiteColor] forState:UIControlStateHighlighted];

    self.trackingButton.buttonColor = [UIColor cloudsColor];
    self.trackingButton.shadowColor = [UIColor silverColor];
    self.trackingButton.shadowHeight = 2.0f;
    self.trackingButton.cornerRadius = 3.0f;
    self.trackingButton.titleLabel.font = [UIFont fontWithName:@"FontAwesome" size:16];
    [self.trackingButton setTitleColor:[UIColor asbestosColor] forState:UIControlStateNormal];
    [self.trackingButton setTitleColor:[UIColor whiteColor] forState:UIControlStateHighlighted];
    [self.trackingButton setTitle:@"\uF124" forState:UIControlStateNormal];

    self.rateButton.buttonColor = [UIColor amethystColor];
    self.rateButton.shadowColor = [UIColor wisteriaColor];
    self.rateButton.shadowHeight = 2.0f;
    self.rateButton.cornerRadius = 3.0f;
    self.rateButton.titleLabel.font = [UIFont fontWithName:@"FontAwesome" size:16];
    [self.rateButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.rateButton setTitleColor:[UIColor whiteColor] forState:UIControlStateHighlighted];
    [self.rateButton setTitle:@"\uF005" forState:UIControlStateNormal];
    
    // Comprobar si hay petición en curso
    [self refreshRide];

    // Refrescar peticion en curso cada 5 segundos
    if (!self.activeRideTimer) {
        self.activeRideTimer = [NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(refreshRide) userInfo:nil repeats:YES];
    }
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    // Observa los cambios de la propiedad userTrackingMode del mapa
    [self.mapView addObserver:self forKeyPath:@"userTrackingMode" options:(NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld) context:nil];
    // Observa los cambios de las propiedades userTrackingEnabled y isShowingAvailability
    [self addObserver:self forKeyPath:@"userTrackingEnabled" options:(NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld) context:nil];
    [self addObserver:self forKeyPath:@"isShowingAvailability" options:(NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld) context:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    // Deja de observar el valor de la propiedad userTrackingMode del mapa
    [self.mapView removeObserver:self forKeyPath:@"userTrackingMode"];
    // Deja de observar los valores de las propiedades userTrackingEnabled y isShowingAvailability
    [self removeObserver:self forKeyPath:@"userTrackingEnabled"];
    [self removeObserver:self forKeyPath:@"isShowingAvailability"];
}

#pragma mark - Key-Value Observing

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    // Si el usuario mueve el mapa, desactivar modo seguimiento (igual que en la app de mapas original)
    if ([keyPath isEqual:@"userTrackingMode"]) {
        if (![change[NSKeyValueChangeNewKey] boolValue]) {
            // El nuevo valor es NO, si el almacenado era YES -> cambiarlo
            if (self.userTrackingEnabled) {
                self.userTrackingEnabled = NO;
            }
        }
    }

    // Controla modo seguimiento del usuario
    if ([keyPath isEqual:@"userTrackingEnabled"]) {
        // Ajusta el modo de seguimiento del mapa
        self.mapView.userTrackingMode = self.userTrackingEnabled ? MKUserTrackingModeFollow : MKUserTrackingModeNone;
        
        // Cambiar estilo botón
        UIColor *buttonColor = self.userTrackingEnabled ? [UIColor peterRiverColor] : [UIColor cloudsColor];
        UIColor *shadowColor = self.userTrackingEnabled ? [UIColor belizeHoleColor] : [UIColor silverColor];
        UIColor *fontColor   = self.userTrackingEnabled ? [UIColor whiteColor]      : [UIColor asbestosColor];
        [self updateAppearanceOfButton:self.trackingButton withButtonColor:buttonColor shadowColor:shadowColor fontColor:fontColor];
    }

    // Controla modo ver disponibilidad
    if ([keyPath isEqual:@"isShowingAvailability"]) {
        // Desactiva seguimiento
        self.userTrackingEnabled = NO;
        
        if (self.isShowingAvailability) {
            // Muestra taxis y actualiza cada 5 segundos
            [self plotPositionsAndZoom:YES];
            if (!self.availabilityTimer) {
                self.availabilityTimer = [NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(plotPositions) userInfo:nil repeats:YES];
            }
        } else {
            // Desactivar posteriores refrescos
            if (self.availabilityTimer) {
                [self.availabilityTimer invalidate];
                self.availabilityTimer = nil;
            }
            
            // Borrar anotaciones
            for (id<MKAnnotation> annotation in self.mapView.annotations) {
                if ([annotation isKindOfClass:[Taxi class]]) {
                    [self.mapView removeAnnotation:annotation];
                }
            }
            
            // Centrar mapa en usuario
            if (self.mapView.userLocationVisible) {
                MKMapPoint userPoint = MKMapPointForCoordinate(self.mapView.userLocation.coordinate);
                MKMapRect userRect = MKMapRectMake(userPoint.x, userPoint.y, 0, 0);
                MKMapRect unionRectThatFits = [self.mapView mapRectThatFits:userRect edgePadding:UIEdgeInsetsFromString(@"{10.0,10.0,10.0,10.0}")];
                [self.mapView setVisibleMapRect:unionRectThatFits animated:YES];
            }
        }
        
        // Cambiar estilo botón
        UIColor *buttonColor = self.isShowingAvailability ? [UIColor carrotColor]  : [UIColor cloudsColor];
        UIColor *shadowColor = self.isShowingAvailability ? [UIColor pumpkinColor] : [UIColor silverColor];
        UIColor *fontColor   = self.isShowingAvailability ? [UIColor whiteColor]   : [UIColor asbestosColor];
        [self updateAppearanceOfButton:self.availabilityButton withButtonColor:buttonColor shadowColor:shadowColor fontColor:fontColor];
    }
}

#pragma mark - Acciones botones interfaz

- (IBAction)toggleAvailability
{
    self.isShowingAvailability = !self.isShowingAvailability;
}

- (IBAction)toggleTracking
{
    self.userTrackingEnabled = !self.userTrackingEnabled;
}

- (IBAction)hailTaxi
{
    // Desactivar botón y mostrar panel
    self.hailButton.enabled = NO;
    [self showRequestPane];
}

#pragma mark - Actualización estilos UI

- (void)updateAppearanceOfButton:(FUIButton *)button withButtonColor:(UIColor *)buttonColor shadowColor:(UIColor *)shadowColor fontColor:(UIColor *)fontColor
{
    // Actualiza colores y fuentes en el hilo principal, tras terminar de procesar el evento actual
    dispatch_async(dispatch_get_main_queue(), ^{
        button.buttonColor = buttonColor;
        button.shadowColor = shadowColor;
        [button setTitleColor:fontColor forState:UIControlStateNormal];
    });
}

#pragma mark - MKMapViewDelegate

- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id<MKAnnotation>)annotation
{
    // Devolver vista para la anotación en el mapa
    if ([annotation isKindOfClass:[MKUserLocation class]]) {
        // Devolviendo nil se usa el punto azul
        return nil;
    } else if ([annotation isKindOfClass:[Taxi class]]) {
        // Taxis disponibles - usar icono de taxi
        static NSString *reuseTaxiId = @"AvailableTaxiMapAnnotation";
        MKAnnotationView *view = [mapView dequeueReusableAnnotationViewWithIdentifier:reuseTaxiId];
        if (!view) {
            view = [[MKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:reuseTaxiId];
            view.canShowCallout = YES;
            view.image = [UIImage imageNamed:@"icon-taxi"];
        }
        return view;
    } else if ([annotation isKindOfClass:[Ride class]]) {
        // Petición en curso, pin verde
        static NSString *reusePinId = @"RidePinMapAnnotation";
        MKPinAnnotationView *view = (MKPinAnnotationView *) [mapView dequeueReusableAnnotationViewWithIdentifier:reusePinId];
        if (!view) {
            view = [[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:reusePinId];
            view.canShowCallout = YES;
            view.pinColor = MKPinAnnotationColorGreen;
            view.animatesDrop = YES;
        }
        return view;
    } else {
        // Nueva petición, pin rojo - permite arrastrar y aparece cayendo con animación
        static NSString *reusePinId = @"CustomerPinMapAnnotation";
        MKPinAnnotationView *view = (MKPinAnnotationView *) [mapView dequeueReusableAnnotationViewWithIdentifier:reusePinId];
        if (!view) {
            view = [[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:reusePinId];
            view.canShowCallout = YES;
            view.pinColor = MKPinAnnotationColorRed;
            view.animatesDrop = YES;
            view.draggable = YES;
        }
        return view;
    }
}

#pragma mark - Disponibilidad

- (void)updateAvailableTaxis:(NSArray *)taxis
{
    // Identificadores taxis disponibles
    NSArray *identifiers = [taxis valueForKey:@"identifier"];

    // Crea diccionario de taxis disponibles por identificador
    NSMutableDictionary *taxisDictionary = [NSMutableDictionary dictionary];
    for (Taxi *taxi in taxis) {
        taxisDictionary[taxi.identifier] = taxi;
    }

    for (id<MKAnnotation> annotation in self.mapView.annotations) {
        if ([annotation isKindOfClass:[Taxi class]]) {
            Taxi *existingTaxi = (Taxi *)annotation;
            if (![identifiers containsObject:existingTaxi.identifier]) {
                // Quitar anotación de taxi que ya no está disponible
                [self.mapView removeAnnotation:annotation];
            } else {
                // Mover anotación de taxi existente
                Taxi *taxi = taxisDictionary[existingTaxi.identifier];
                [UIView animateWithDuration:0.5 animations:^{
                    [existingTaxi willChangeValueForKey:@"coordinate"];
                    existingTaxi.latitude = taxi.latitude;
                    existingTaxi.longitude = taxi.longitude;
                    [existingTaxi didChangeValueForKey:@"coordinate"];
                }];
                // Quitar taxi tratado del diccionario de disponibles
                [taxisDictionary removeObjectForKey:existingTaxi.identifier];
            }
        }
    }

    // Añadir taxis disponibles nuevos al mapa
    for (NSString *identifier in taxisDictionary) {
        [self.mapView addAnnotation:taxisDictionary[identifier]];
    }
}

- (void)plotPositions
{
    // Actualiza taxis disponibles sin hacer zoom en el mapa
    // Solo se hace zoom la primera vez tras pulsar el botón
    // El resto de veces se ejecuta este método
    [self plotPositionsAndZoom:NO];
}

- (void)plotPositionsAndZoom:(BOOL)zoom
{
    // Carga taxis disponibles
    [Taxi availableTaxisWithSuccess:^(NSArray *taxis) {
        [self updateAvailableTaxis:taxis];

        if (zoom) {
            // Hacer zoom en el mapa para que se vean todos
            MKMapRect zoomRect = MKMapRectNull;
            for (id <MKAnnotation> annotation in self.mapView.annotations)
            {
                MKMapPoint annotationPoint = MKMapPointForCoordinate(annotation.coordinate);
                MKMapRect pointRect = MKMapRectMake(annotationPoint.x, annotationPoint.y, 0.1, 0.1);
                zoomRect = MKMapRectUnion(zoomRect, pointRect);
            }

            // También se tiene que mostrar la posición del usuario
            if (self.mapView.userLocationVisible) {
                MKMapPoint userPoint = MKMapPointForCoordinate(self.mapView.userLocation.coordinate);
                MKMapRect userRect = MKMapRectMake(userPoint.x, userPoint.y, 0, 0);
                zoomRect = MKMapRectUnion(zoomRect, userRect);
            }

            // Dejar un poco de padding en la región a mostrar en el mapa y animar
            MKMapRect unionRectThatFits = [self.mapView mapRectThatFits:zoomRect edgePadding:UIEdgeInsetsFromString(@"{10.0,10.0,10.0,10.0}")];
            [self.mapView setVisibleMapRect:unionRectThatFits animated:YES];
        }
    } failure:nil];
}

#pragma mark - Solicitud de taxi

- (void)showRequestPane
{
    // Desactiva mostrar taxis disponibles
    self.isShowingAvailability = NO;

    // Crea pin con la posición del usuario
    self.customerRequest = [[Request alloc] init];
    self.customerRequest.coordinate = self.mapView.userLocation.location.coordinate;

    // Activa botones del panel
    self.requestRideButton.enabled = YES;
    self.cancelRideButton.enabled = YES;    
    // Muestra panel con animación desde abajo
    CGRect sliderFrame = self.requestRideView.frame;
    sliderFrame.origin.y = self.view.frame.size.height;
    self.requestRideView.frame = sliderFrame;
    self.requestRideView.hidden = NO;
    sliderFrame.origin.y -= sliderFrame.size.height;
    [UIView animateWithDuration:0.5 animations:^{
        self.requestRideView.frame = sliderFrame;
    }];

    // Centra el mapa en la posición del usuario
    UIEdgeInsets padding = UIEdgeInsetsMake(10, 10, sliderFrame.size.height, 10);
    MKMapPoint userPoint = MKMapPointForCoordinate(self.customerRequest.coordinate);
    MKMapRect userRect = MKMapRectMake(userPoint.x - 10, userPoint.y - 10, 20, 20);
    MKMapRect unionRectThatFits = [self.mapView mapRectThatFits:userRect edgePadding:padding];
    [self.mapView setVisibleMapRect:unionRectThatFits edgePadding:padding animated:YES];

    // Animación caída del pin
    double delayInSeconds = 1.5;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        if (self.customerRequest) {
            [self.mapView addAnnotation:self.customerRequest];
            [self.mapView selectAnnotation:self.customerRequest animated:YES];
        }
    });
}

- (void)hideRequestPane
{
    // Desactiva botones
    self.requestRideButton.enabled = NO;
    self.cancelRideButton.enabled = NO;

    // Mueve panel fuera de la pantalla
    CGRect sliderFrame = self.requestRideView.frame;
    sliderFrame.origin.y = self.view.frame.size.height;
    [UIView animateWithDuration:0.5 animations:^{
        self.requestRideView.frame = sliderFrame;
    } completion:^(BOOL finished) {
        self.hailButton.enabled = YES;
        if (!self.activeRideTimer) {
            self.activeRideTimer = [NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(refreshRide) userInfo:nil repeats:YES];
        }
        [self refreshRide];
        self.requestRideView.hidden = YES;
    }];
}

- (IBAction)cancelRequest
{
    // Quitar pin del mapa y ocultar panel
    [self.mapView removeAnnotation:self.customerRequest];
    self.customerRequest = nil;

    [self hideRequestPane];
}

#define SECONDS_PER_HOUR 3600

- (IBAction)confirmRequest
{
    if (self.rideTimeControl.selectedSegmentIndex == 0) {
        // Reservar ahora
        [self.customerRequest hailTaxiWithSuccess:^{
            // Guardar localizador y quitar pin
            [[LUKeychainAccess standardKeychainAccess] setString:self.customerRequest.locator forKey:@"locator"];
            [self.mapView removeAnnotation:self.customerRequest];
            [self performSelector:@selector(setCustomerRequest:) withObject:nil afterDelay:0];
            [self hideRequestPane];
            // Refrescar información
            [self refreshRide];
        } failure:nil];
    } else {
        NSDate *requestTime = nil;
        if (self.rideTimeControl.selectedSegmentIndex == 1) {
            // Reservar dentro de 1 hora
            requestTime = [NSDate dateWithTimeIntervalSinceNow:(1 * SECONDS_PER_HOUR)];
        } else {
            // Reservar dentro de 2 horas
            requestTime = [NSDate dateWithTimeIntervalSinceNow:(2 * SECONDS_PER_HOUR)];
        }
        [self.customerRequest schedulePickupAt:requestTime success:^{
            // Guardar localizador y quitar pin
            [[LUKeychainAccess standardKeychainAccess] setString:self.customerRequest.locator forKey:@"locator"];
            [self.mapView removeAnnotation:self.customerRequest];
            [self performSelector:@selector(setCustomerRequest:) withObject:nil afterDelay:0];
            [self hideRequestPane];
            // Refrescar información
            [self refreshRide];
        } failure:nil];
    }
}

#pragma mark - Trayecto en curso

- (void)refreshRide
{
    // Actualiza información del trayecto en curso
    NSString *locator = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"locator"];

    if (locator) {
        // Si hay locator, no mostrar taxis disponibles
        if (self.isShowingAvailability) {
            self.isShowingAvailability = NO;
        }

        [Ride activeRideWithLocator:locator success:^(Ride *ride) {
            if (ride) {
                if (ride.status == TaxiRideStatusComplete) {
                    // Borrar locator (no se necesita para puntuar)
                    [[LUKeychainAccess standardKeychainAccess] setData:nil forKey:@"locator"];
                    if (self.activeRideTimer) {
                        [self.activeRideTimer invalidate];
                        self.activeRideTimer = nil;
                    }
                    [self hideCurrentRidePane];
                    // Quitar taxi del mapa
                    if (self.assignedTaxi) {
                        [self.mapView removeAnnotation:self.assignedTaxi];
                        self.assignedTaxi = nil;
                    }
                } else if (ride.status == TaxiRideStatusRated) {
                    // Borrar locator, ya se ha hecho todo
                    [[LUKeychainAccess standardKeychainAccess] setData:nil forKey:@"locator"];
                    if (self.activeRideTimer) {
                        [self.activeRideTimer invalidate];
                        self.activeRideTimer = nil;
                    }
                    [self hideCurrentRidePane];
                    // Quitar taxi del mapa
                    if (self.assignedTaxi) {
                        [self.mapView removeAnnotation:self.assignedTaxi];
                        self.assignedTaxi = nil;
                    }
                } else {
                    // Peticion activa
                    if (!self.activeRide) {
                        self.activeRide = ride;
                    }
                    [self showCurrentRidePane];
                    self.activeRide.status = ride.status;
                    [Taxi assignedTaxiWithLocator:locator success:^(Taxi *taxi) {
                        if (self.assignedTaxi && taxi) {
                            // Ya había taxi asignado, animar posición en mapa
                            [UIView animateWithDuration:0.5 animations:^{
                                [self.assignedTaxi willChangeValueForKey:@"coordinate"];
                                self.assignedTaxi.latitude = taxi.latitude;
                                self.assignedTaxi.longitude = taxi.longitude;
                                [self.assignedTaxi didChangeValueForKey:@"coordinate"];
                            }];
                        } else if (taxi) {
                            // No había taxi asignado, añadirlo al mapa
                            self.assignedTaxi = taxi;
                            if (taxi) {
                                [self showAssignedTaxi];
                            }
                        }
                    } failure:nil];
                }

                // Actualiza etiquetas
                [self updateCurrentRideLabels];
            } else {
                // No hay viaje en curso para este localizador, borrarlo
                [[LUKeychainAccess standardKeychainAccess] setData:nil forKey:@"locator"];
                if (self.activeRideTimer) {
                    [self.activeRideTimer invalidate];
                    self.activeRideTimer = nil;
                }
                [self hideCurrentRidePane];
                // Quitar taxi del mapa
                if (self.assignedTaxi) {
                    [self.mapView removeAnnotation:self.assignedTaxi];
                    self.assignedTaxi = nil;
                }
            }
        } failure:nil];
    }

    // Comprueba si hay viajes pendientes de puntuar
    [Ride unratedRideWithSuccess:^(Ride *ride) {
        // Hay viaje sin puntuar, muestra el botón
        if (ride) {
            self.unratedRide = ride;
            self.rateButton.hidden = NO;
        }
    } failure:^(NSError *error) {
        // No hay viaje sin puntuar, oculta el botón
        self.unratedRide = nil;
        self.rateButton.hidden = YES;
    }];
}

- (void)updateCurrentRideLabels
{
    // Actualiza etiqueta estado
    switch (self.activeRide.status) {
        case TaxiRideStatusPending:
        case TaxiRideStatusAssigned:
            self.activeRideStatusLabel.text = NSLocalizedString(@"Solicitud pendiente", @"Mensaje de solicitud de viaje pendiente de asignación");
            break;

        case TaxiRideStatusAccepted:
            self.activeRideStatusLabel.text = NSLocalizedString(@"Taxista en camino", @"Mensaje de solicitud de viaje aceptada por taxista");
            break;

        case TaxiRideStatusEnRoute:
            self.activeRideStatusLabel.text = NSLocalizedString(@"Trayecto en curso", @"Mensaje de viaje iniciado");
            break;

        default:
            break;
    }

    // Si hay taxi asignado, actualiza etiqueta distancia
    if (self.assignedTaxi && self.mapView.userLocation) {
        CLLocation *taxiLocation = [[CLLocation alloc] initWithLatitude:self.assignedTaxi.coordinate.latitude longitude:self.assignedTaxi.coordinate.longitude];
        CLLocationDistance distance = [self.mapView.userLocation.location distanceFromLocation:taxiLocation];
        self.activeRideDistanceLabel.text = [Ride stringWithDistance:distance];
    } else {
        self.activeRideDistanceLabel.text = @"";
    }
}

- (void)showCurrentRidePane
{
    // Si ya está visible no hacer nada
    if (!self.isShowingCurrentRide) {
        self.isShowingCurrentRide = YES;
        // Actualiza etiquetas del panel
        [self updateCurrentRideLabels];

        // Muestra panel con animación desde abajo
        CGRect sliderFrame = self.activeRideView.frame;
        sliderFrame.origin.y = self.view.frame.size.height;
        self.activeRideView.frame = sliderFrame;
        self.activeRideView.hidden = NO;
        sliderFrame.origin.y -= sliderFrame.size.height;

        [UIView animateWithDuration:0.5 animations:^{
            self.activeRideView.frame = sliderFrame;
        }];

        // Añade pin con la posición de la solicitud
        if (self.activeRide) {
            [self.mapView addAnnotation:self.activeRide];
            [self.mapView selectAnnotation:self.activeRide animated:YES];
        }

        // Centra el mapa en la posición del usuario
        UIEdgeInsets padding = UIEdgeInsetsMake(10, 10, sliderFrame.size.height, 10);
        MKMapPoint userPoint = MKMapPointForCoordinate(self.activeRide.coordinate);
        MKMapRect userRect = MKMapRectMake(userPoint.x - 10, userPoint.y - 10, 20, 20);
        MKMapRect unionRectThatFits = [self.mapView mapRectThatFits:userRect edgePadding:padding];
        [self.mapView setVisibleMapRect:unionRectThatFits edgePadding:padding animated:YES];
    }
}

- (void)hideCurrentRidePane
{
    // Si ya está oculto no hacer nada
    if (self.isShowingCurrentRide) {
        self.isShowingCurrentRide = NO;

        // Quitar pin
        [self.mapView removeAnnotation:self.activeRide];
        self.activeRide = nil;

        // Mueve panel fuera de la pantalla
        CGRect sliderFrame = self.activeRideView.frame;
        sliderFrame.origin.y = self.view.frame.size.height;
        [UIView animateWithDuration:0.5 animations:^{
            self.activeRideView.frame = sliderFrame;
        } completion:^(BOOL finished) {
            [self refreshRide];
            self.activeRideView.hidden = YES;
        }];
    }
}

- (void)showAssignedTaxi
{
    // Muestra taxi asignado en mapa, hace zoom para que se vean el taxi y el usuario
    [self.mapView addAnnotation:self.assignedTaxi];
    MKMapRect zoomRect = MKMapRectNull;
    for (id <MKAnnotation> annotation in self.mapView.annotations)
    {
        MKMapPoint annotationPoint = MKMapPointForCoordinate(annotation.coordinate);
        MKMapRect pointRect = MKMapRectMake(annotationPoint.x, annotationPoint.y, 0.1, 0.1);
        zoomRect = MKMapRectUnion(zoomRect, pointRect);
    }
    
    if (self.mapView.userLocationVisible) {
        MKMapPoint userPoint = MKMapPointForCoordinate(self.mapView.userLocation.coordinate);
        MKMapRect userRect = MKMapRectMake(userPoint.x, userPoint.y, 0, 0);
        zoomRect = MKMapRectUnion(zoomRect, userRect);
    }
    MKMapRect unionRectThatFits = [self.mapView mapRectThatFits:zoomRect edgePadding:UIEdgeInsetsFromString(@"{10.0,10.0,10.0,10.0}")];
    [self.mapView setVisibleMapRect:unionRectThatFits animated:YES];
}

#pragma mark - Puntuar trayecto

- (void)showRateRidePane
{
    // Inicializa puntuación a cero
    self.rateControl.rating = 0;
    [self.rateControl ratingDidChange];

    // Muestra panel con animación desde abajo
    CGRect sliderFrame = self.ratePaneView.frame;
    sliderFrame.origin.y = self.view.frame.size.height;
    self.ratePaneView.frame = sliderFrame;
    self.ratePaneView.hidden = NO;
    sliderFrame.origin.y -= sliderFrame.size.height;

    [UIView animateWithDuration:0.5 animations:^{
        self.ratePaneView.frame = sliderFrame;
    }];
}

- (void)hideRateRidePane
{
    // Mueve panel fuera de la pantalla
    CGRect sliderFrame = self.ratePaneView.frame;
    sliderFrame.origin.y = self.view.frame.size.height;
    [UIView animateWithDuration:0.5 animations:^{
        self.ratePaneView.frame = sliderFrame;
    } completion:^(BOOL finished) {
        self.ratePaneView.hidden = YES;
        [self refreshRide];
    }];
}

- (IBAction)rateLastRide
{
    // Muestra panel de puntuación y desactiva botón de puntuar
    self.rateButton.enabled = NO;
    [self showRateRidePane];
}

- (IBAction)confirmRate
{
    // Guarda puntuación vía API, oculta panel y oculta botón de puntuar
    // El control guarda la puntuación en una escala de 100 - en la API se usa de 1 a 5
    int rating = self.rateControl.rating / 20;
    [self.unratedRide rateRideWithRating:@(rating) success:^{
        [self hideRateRidePane];
        self.rateButton.enabled = YES;
        self.rateButton.hidden = YES;
    } failure:nil];
}

- (IBAction)cancelRate
{
    // Cancela puntuación, oculta panel y vuelve a activar botón de puntuar
    [self hideRateRidePane];
    self.rateButton.enabled = YES;
}

@end
