//
//  Request.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "Request.h"
#import <AddressBookUI/AddressBookUI.h>
#import "TaxiAPIClient.h"


@interface Request ()
@property (nonatomic, strong) NSString *address;
@end

@implementation Request

// Hace un geocoding inverso para obtener la dirección a partir de la coordenada
- (void)updateAddress
{
    CLGeocoder *geocoder = [[CLGeocoder alloc] init];
    CLLocation *location = [[CLLocation alloc] initWithLatitude:self.coordinate.latitude longitude:self.coordinate.longitude];
    [geocoder reverseGeocodeLocation:location completionHandler:^(NSArray *placemarks, NSError *error) {
        if (placemarks && placemarks.count > 0) {
            MKPlacemark *placemark = placemarks[0];

            [self willChangeValueForKey:@"subtitle"];
            self.address = ABCreateStringWithAddressDictionary(placemark.addressDictionary, NO);
            [self didChangeValueForKey:@"subtitle"];
        }
    }];
}

#pragma mark - MKAnnotation

- (NSString *)title
{
    return NSLocalizedString(@"Aquí", @"Etiqueta solicitud en mapa");
}

- (NSString *)subtitle
{
    return self.address;
}

- (void)setCoordinate:(CLLocationCoordinate2D)coordinate
{
    _coordinate = coordinate;
    [self updateAddress];
}

#pragma mark - Interfaz con API

- (void)hailTaxiWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] hailTaxiWithCoordinate:self.coordinate success:^(id JSON) {
        self.locator = JSON[@"locator"];
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

- (void)schedulePickupAt:(NSDate *)time success:(void (^)(void))success failure:(void (^)(NSError *))failure
{
    [[TaxiAPIClient sharedClient] schedulePickupWithCoordinate:self.coordinate atTime:time success:^(id JSON) {
        self.locator = JSON[@"locator"];
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}


@end
