//
//  Ride.h
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <MapKit/MapKit.h>

typedef NS_ENUM(NSUInteger, TaxiRideStatus) {
    TaxiRideStatusPending,
    TaxiRideStatusAssigned,
    TaxiRideStatusAccepted,
    TaxiRideStatusEnRoute,
    TaxiRideStatusComplete,
    TaxiRideStatusRated
};

@interface Ride : NSObject <MKAnnotation>
@property (nonatomic, copy) NSString *locator;
@property (nonatomic, copy) NSDate *bookingTime;
@property (nonatomic, copy) NSNumber *latitude;
@property (nonatomic, copy) NSNumber *longitude;
@property (nonatomic, assign) CLLocationDistance distance;
@property (nonatomic, assign) TaxiRideStatus status;
@property (nonatomic, strong) NSString *address;
@property (nonatomic, strong) MKPlacemark *placemark;

- (id)initWithAttributes:(NSDictionary *)attributes;
- (void)updateDistanceFromLocation:(CLLocation *)location;
- (NSString *)distanceDescription;
- (void)beginRideWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure;
- (void)endRideWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure;
- (void)rateRideWithRating:(NSNumber *)rating success:(void (^)(void))success failure:(void (^)(NSError *error))failure;
- (void)updateAddress;
- (NSString *)statusDisplayName;

+ (void)activeRideWithLocator:(NSString *)locator success:(void (^)(Ride *ride))success failure:(void (^)(NSError *error))failure;
+ (void)unratedRideWithSuccess:(void (^)(Ride *ride))success failure:(void (^)(NSError *error))failure;
+ (NSString *)stringWithDistance:(double)distance;
@end
