//
//  Taxi.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "Taxi.h"
#import "TaxiAPIClient.h"
#import "TaxiLocationDelegate.h"
#import <LUKeychainAccess/LUKeychainAccess.h>

@implementation Taxi

- (id)initWithAttributes:(NSDictionary *)attributes
{
    self = [self init];

    if (self){
        [self setValue:attributes[@"identifier"] forKey:@"identifier"];
        [self setValue:attributes[@"latitude"] forKey:@"latitude"];
        [self setValue:attributes[@"longitude"] forKey:@"longitude"];
        [self setValue:attributes[@"available"] forKey:@"available"];
    }

    return self;
}

#pragma mark - Interfaz con API

// Actualizar posición del taxi actual en servidor
+ (void)didChangeLocation:(CLLocation *)location
{
    if (location) {
        [[TaxiAPIClient sharedClient] updateTaxiPositionWithCoordinate:location.coordinate success:^(id JSON) {
            NSLog(@"Position updated");
        } failure:nil];
    }
}

// Actualizar token del taxi actual en servidor
+ (void)updateDeviceToken
{
    [[TaxiAPIClient sharedClient] updateTaxiDeviceWithSuccess:nil failure:nil];
}

// Obtiene taxi para el usuario actual
+ (void)fetchWithSuccess:(void (^)(Taxi *))success failure:(void (^)(NSError *))failure
{
    [[TaxiAPIClient sharedClient] getTaxiInformationWithSuccess:^(id JSON){
        Taxi *taxi = [[Taxi alloc] initWithAttributes:JSON];
        if (success) {
            success(taxi);
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Obtiene taxis disponibles
+ (void)availableTaxisWithSuccess:(void (^)(NSArray *))success failure:(void (^)(NSError *))failure
{
    [[TaxiAPIClient sharedClient] getAvailableTaxisWithSuccess:^(id JSON){

        NSMutableArray *mutableTaxis = [NSMutableArray arrayWithCapacity:[JSON count]];
        for (NSDictionary *attributes in JSON) {
            Taxi *taxi = [[Taxi alloc] initWithAttributes:attributes];
            [mutableTaxis addObject:taxi];
        }
        if (success) {
            success([NSArray arrayWithArray:mutableTaxis]);
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Obtiene taxi asignado al localizador indicado
+ (void)assignedTaxiWithLocator:(NSString *)locator success:(void (^)(Taxi *))success failure:(void (^)(NSError *))failure
{
    [[TaxiAPIClient sharedClient] getAssignedTaxiWithLocator:locator success:^(id JSON){
        Taxi *taxi = nil;
        if ([JSON count] > 0) {
            taxi = [[Taxi alloc] initWithAttributes:JSON];
        }
        if (success) {
            success(taxi);
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

#pragma mark - MKAnnotation

- (NSString *)title {
    return [NSString stringWithFormat:@"Taxi '%@'", self.identifier];
}

- (NSString *)subtitle {
    return nil; //_address;
}

- (CLLocationCoordinate2D)coordinate {
    CLLocationDegrees lat = [self.latitude doubleValue];
    CLLocationDegrees lon = [self.longitude doubleValue];
    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(lat, lon);
    if (NO == CLLocationCoordinate2DIsValid(coord))
        NSLog(@"Invalid Centroid: lat=%lf lon=%lf", lat, lon);
    return coord;
}

@end
