//
//  TaxiAppDelegate.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiAppDelegate.h"
#import <AudioToolbox/AudioToolbox.h>
#import <AFNetworking.h>
#import <LUKeychainAccess.h>
#import <FUIAlertView.h>
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>
#import "Taxi.h"
#import "TaxiAPIClient.h"


@implementation TaxiAppDelegate

#define IS_IPHONE_5 ( fabs( ( double )[ [ UIScreen mainScreen ] bounds ].size.height - ( double )568 ) < DBL_EPSILON )

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Registrarse para notificaciones push
    [[UIApplication sharedApplication] registerForRemoteNotificationTypes:(UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeSound)];

    // Limpiar notificaciones pendientes
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];

    // Configurar el indicador de actividad de AFNetworking
    [AFNetworkActivityIndicatorManager sharedManager].enabled = YES;

    // Mostrar splash screen mientras se comprueba el login
    UIImageView *splash;
    if(IS_IPHONE_5) {
        splash = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"Default-568h.png"]];
    } else {
        splash = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"Default.png"]];
    }
    [self.window.rootViewController.view addSubview:splash];
    
    // Hace transición de un segundo
    [UIView animateWithDuration:1.0
                     animations:^{
                         splash.alpha = 0;
                     }
                     completion:^(BOOL finished) {
                         [splash removeFromSuperview];
                     }];

    return YES;
}

#pragma mark - Notificaciones Push

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    // Guardar token de dispositivo
    [[LUKeychainAccess standardKeychainAccess] setString:[deviceToken description] forKey:@"device"];
    [Taxi updateDeviceToken];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    NSLog(@"Error: %@", error);
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
    if (application.applicationState == UIApplicationStateActive) {
        // 1007 es el sonido por defecto en iOS al recibir SMS (tritono)
        AudioServicesPlaySystemSound(1007);
        
        FUIAlertView *alertView = [[FUIAlertView alloc] initWithTitle:NSLocalizedString(@"Aviso", @"Título aviso AlertView")
                                                              message:NSLocalizedString(userInfo[@"aps"][@"alert"][@"loc-key"], nil)
                                                             delegate:nil cancelButtonTitle:NSLocalizedString(@"Cerrar", @"Botón Cerrar AlertView")
                                                    otherButtonTitles:nil];
        alertView.titleLabel.textColor = [UIColor cloudsColor];
        alertView.titleLabel.font = [UIFont boldFlatFontOfSize:16];
        alertView.messageLabel.textColor = [UIColor cloudsColor];
        alertView.messageLabel.font = [UIFont flatFontOfSize:14];
        alertView.backgroundOverlay.backgroundColor = [[UIColor cloudsColor] colorWithAlphaComponent:0.5];
        alertView.alertContainer.backgroundColor = [UIColor midnightBlueColor];
        alertView.defaultButtonColor = [UIColor cloudsColor];
        alertView.defaultButtonShadowColor = [UIColor asbestosColor];
        alertView.defaultButtonFont = [UIFont boldFlatFontOfSize:16];
        alertView.defaultButtonTitleColor = [UIColor asbestosColor];
        [alertView show];
    }
    // Si no estaba activa, viene de cualquier otro estado.... normalmente inactivo
    // UIApplicationStateBackground o UIApplicationStateInactive

    // Limpiar notificaciones
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];

    // Recargar datos
    [[NSNotificationCenter defaultCenter] postNotificationName:@"UPDATE_DATA" object:nil userInfo:nil];
}

@end
