//
//  TaxiAssignmentViewController.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiAssignmentViewController.h"
#import "TaxiAPIClient.h"
#import <MapKit/MapKit.h>
#import <UIFont+FlatUI.h>
#import <UIColor+FlatUI.h>
#import <FUIButton.h>
#import <AddressBookUI/AddressBookUI.h>
#import "Ride.h"
#import <SVProgressHUD.h>

@interface TaxiAssignmentViewController ()

@property (strong, nonatomic) IBOutletCollection(UILabel) NSArray *allLabels;
@property (weak, nonatomic) IBOutlet FUIButton *acceptButton;
@property (weak, nonatomic) IBOutlet FUIButton *rejectButton;
@property (weak, nonatomic) IBOutlet MKMapView *positionMapView;
@property (weak, nonatomic) IBOutlet UILabel *addressLabel;
@property (weak, nonatomic) IBOutlet UILabel *distanceLabel;
- (IBAction)acceptAssignment;
- (IBAction)rejectAssignment;

@end

@implementation TaxiAssignmentViewController

#pragma mark - Lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.view.backgroundColor = [UIColor cloudsColor];
    
    // Configurar Flat UI
    [self.allLabels makeObjectsPerformSelector:@selector(setFont:) withObject:[UIFont flatFontOfSize:16]];
    [self.addressLabel setFont:[UIFont boldFlatFontOfSize:26]];
    self.addressLabel.text = @"";
    [self.distanceLabel setFont:[UIFont boldFlatFontOfSize:20]];
    self.distanceLabel.text = self.currentAssignment.ride.distanceDescription;
    self.distanceLabel.textColor = [UIColor nephritisColor];

    self.acceptButton.buttonColor = [UIColor emerlandColor];
    self.acceptButton.shadowColor = [UIColor nephritisColor];
    self.acceptButton.shadowHeight = 3.0f;
    self.acceptButton.cornerRadius = 6.0f;
    self.acceptButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.acceptButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.acceptButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];
    
    self.rejectButton.buttonColor = [UIColor alizarinColor];
    self.rejectButton.shadowColor = [UIColor pomegranateColor];
    self.rejectButton.shadowHeight = 3.0f;
    self.rejectButton.cornerRadius = 6.0f;
    self.rejectButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.rejectButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.rejectButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];

    [self.navigationItem setHidesBackButton:YES];
    [self.positionMapView addAnnotation:self.currentAssignment.ride];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    // Crear map points
    MKMapPoint userPoint = MKMapPointForCoordinate(self.positionMapView.userLocation.coordinate);
    MKMapPoint customerPoint = MKMapPointForCoordinate(self.currentAssignment.ride.coordinate);
    // Crear map rects con tamaño 0
    MKMapRect userRect = MKMapRectMake(userPoint.x, userPoint.y, 0, 0);
    MKMapRect customerRect = MKMapRectMake(customerPoint.x, customerPoint.y, 0, 0);
    // Crear la unión de ambos rects
    MKMapRect unionRect = MKMapRectUnion(userRect, customerRect);
    MKMapRect unionRectThatFits = [self.positionMapView mapRectThatFits:unionRect edgePadding:UIEdgeInsetsFromString(@"{10.0,10.0,10.0,10.0}")];
    [self.positionMapView setVisibleMapRect:unionRectThatFits animated:YES];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    // Observa cambios en la dirección y la distancia de la solicitud
    [self.currentAssignment.ride addObserver:self forKeyPath:@"address" options:NSKeyValueObservingOptionNew context:nil];
    [self.currentAssignment.ride addObserver:self forKeyPath:@"distance" options:NSKeyValueObservingOptionNew context:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];

    // Deja de observar cambios
    [self.currentAssignment.ride removeObserver:self forKeyPath:@"address"];
    [self.currentAssignment.ride removeObserver:self forKeyPath:@"distance"];
}

#pragma mark - Key-Value Observing

// Si hay cambios en dirección o distancia actualiza las etiquetas
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if ([keyPath isEqual:@"address"]) {
        self.addressLabel.text = self.currentAssignment.ride.address;
    }
    if ([keyPath isEqual:@"distance"]) {
        self.distanceLabel.text = self.currentAssignment.ride.distanceDescription;
    }
}

#pragma mark - Acciones

- (IBAction)acceptAssignment {
    [self.currentAssignment acceptAssignmentWithSuccess:^{
        [SVProgressHUD showSuccessWithStatus:NSLocalizedString(@"Solicitud aceptada", @"Mensaje tras aceptar solicitud")];
        [self.navigationController popViewControllerAnimated:YES];
    } failure:nil];
}

- (IBAction)rejectAssignment {
    [self.currentAssignment rejectAssignmentWithSuccess:^{
        [SVProgressHUD showErrorWithStatus:NSLocalizedString(@"Solicitud rechazada", @"Mensaje tras rechazar solicitud")];
        [self.navigationController popViewControllerAnimated:YES];
    } failure:nil];
}

@end
