//
//  TaxiLocationDelegate.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiLocationDelegate.h"

@interface TaxiLocationDelegate ()
@property (nonatomic, strong) CLLocationManager *locationManager;
@end

@implementation TaxiLocationDelegate

// Singleton para que sólo haya una instancia
+ (TaxiLocationDelegate *)sharedLocationDelegate {
    static TaxiLocationDelegate *_sharedLocationDelegate = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedLocationDelegate = [[self alloc] init];
    });
    
    return _sharedLocationDelegate;
}

- (CLLocationManager *)locationManager
{
    if (!_locationManager) {
        _locationManager = [[CLLocationManager alloc] init];
        _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        _locationManager.distanceFilter = 10;
        _locationManager.delegate = self;
    }

    return _locationManager;
}

#pragma mark - Interfaz pública

// Observar posición del usuario
- (void)startUpdatingLocation
{
    [self.locationManager startUpdatingLocation];
}

- (void)stopUpdatingLocation
{
    [self.locationManager stopUpdatingLocation];
}

// Observar regiones
- (void)startMonitoringRegionWithLocation:(CLLocationCoordinate2D)location radius:(CLLocationDistance)radius identifier:(NSString *)identifier
{
    [self stopMonitoringRegionWithIdentifier:identifier];

    CLRegion *newRegion = [[CLRegion alloc] initCircularRegionWithCenter:location radius:radius identifier:identifier];
    [self.locationManager startMonitoringForRegion:newRegion];
}

- (void)stopMonitoringRegionWithIdentifier:(NSString *)identifier
{
    NSArray *regions = [self.locationManager.monitoredRegions allObjects];
    for (CLRegion *region in regions) {
        if ([region.identifier isEqualToString:identifier]) {
            [self.locationManager stopMonitoringForRegion:region];
        }
    }    
}

# pragma mark - CLLocationManagerDelegate

// Si se ha movido el usuario
- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray *)locations
{
    self.lastLocation = [locations lastObject];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DID_CHANGE_LOCATION" object:nil userInfo:@{@"location" : self.lastLocation}];
}

// Si se ha entrado en una región definida
- (void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DID_ENTER_REGION" object:nil userInfo:@{@"region" : region.identifier}];
}


@end
