//
//  TaxiReputationViewController.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiReputationViewController.h"
#import "TaxiAPIClient.h"
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>
#import <UIBarButtonItem+FlatUI.h>
#import <LUKeychainAccess.h>
#import <SVProgressHUD.h>
#import "Reputation.h"

@interface TaxiReputationViewController ()
@property (strong, nonatomic) IBOutletCollection(UILabel) NSArray *allLabels;
@property (strong, nonatomic) IBOutletCollection(UILabel) NSArray *allStarsLabels;

@property (weak, nonatomic) IBOutlet UILabel *averageLabel;
@property (weak, nonatomic) IBOutlet UILabel *fiveStarRatingsLabel;
@property (weak, nonatomic) IBOutlet UILabel *fourStarRatingsLabel;
@property (weak, nonatomic) IBOutlet UILabel *threeStarRatingsLabel;
@property (weak, nonatomic) IBOutlet UILabel *twoStarRatingsLabel;
@property (weak, nonatomic) IBOutlet UILabel *oneStarRatingsLabel;
@end

@implementation TaxiReputationViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.view.backgroundColor = [UIColor cloudsColor];

    // Configurar Flat UI
    [self.allLabels makeObjectsPerformSelector:@selector(setFont:) withObject:[UIFont flatFontOfSize:16]];
    [self.allStarsLabels makeObjectsPerformSelector:@selector(setFont:) withObject:[UIFont boldFlatFontOfSize:16]];
    [self.allStarsLabels makeObjectsPerformSelector:@selector(setTextColor:) withObject:[UIColor orangeColor]];

    self.averageLabel.font = [UIFont boldFlatFontOfSize:50];
    self.averageLabel.textColor = [UIColor emerlandColor];

    self.averageLabel.text = @"";
    self.fiveStarRatingsLabel.text = @"";
    self.fourStarRatingsLabel.text = @"";
    self.threeStarRatingsLabel.text = @"";
    self.twoStarRatingsLabel.text = @"";
    self.oneStarRatingsLabel.text = @"";
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    [SVProgressHUD showWithStatus:NSLocalizedString(@"Actualizando...", @"Mensaje de actualización de información en progreso") maskType:SVProgressHUDMaskTypeBlack];

    // Obtiene información y actualiza las etiquetas
    [Reputation fetchWithSuccess:^(Reputation *reputation){
        // Muestra media con un decimal
        NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
        [numberFormatter setLocale:[NSLocale currentLocale]];
        [numberFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
        [numberFormatter setMaximumFractionDigits:1];

        self.averageLabel.text = [numberFormatter stringFromNumber:reputation.average];
        self.oneStarRatingsLabel.text      = [reputation.countsByValue[@1] stringValue];
        self.twoStarRatingsLabel.text      = [reputation.countsByValue[@2] stringValue];
        self.threeStarRatingsLabel.text    = [reputation.countsByValue[@3] stringValue];
        self.fourStarRatingsLabel.text     = [reputation.countsByValue[@4] stringValue];
        self.fiveStarRatingsLabel.text     = [reputation.countsByValue[@5] stringValue];
        
        [SVProgressHUD dismiss];
    } failure:nil];
}

@end
