# encoding : utf-8
module Api
  module V1
    class CustomerController < ApplicationController
      respond_to :json

      # comprueba disponibilidad antes de poder hacer una solicitud
      before_filter :check_availability, only: [ :hail_taxi, :schedule_pickup ]

      NO_AVAILABILITY = "NO_AVAILABILITY"

      # obtener taxis disponibles
      def available_taxis
        @taxis = Taxi.available
      end

      # pedir un taxi
      def hail_taxi
        device = params[:device]
        lat = params[:latitude]
        long = params[:longitude]
        @booking = Ride.create(device: device, latitude: lat, longitude: long)
        # se asigna inmediatamente
        @booking.assign_taxi
      end

      # solicitar un taxi para la hora indicada
      # se asignara mediante tarea programada
      def schedule_pickup
        device = params[:device]
        lat = params[:latitude]
        long = params[:longitude]
        time = params[:booking_time]
        @booking = Ride.create(device: device, latitude: lat, longitude: long, booking_time: time)
      end

      # obtener informacion del taxi asignado al cliente
      def assigned_taxi
        device = params[:device]
        locator = params[:locator]
        @taxi = Ride.for_device_and_locator(device, locator).accepted_assignment.taxi rescue nil
      end

      # obtener informacion de la solicitud realizada por el cliente
      def requested_ride
        device = params[:device]
        locator = params[:locator]
        @ride = Ride.for_device_and_locator(device, locator)
      end

      # obtener informacion del ultimo servicio completado aun sin puntuar
      def unrated
        device = params[:device]
        @ride = Ride.current_unrated_for_device(device)
      end

      # puntuar servicio indicado por el localizador
      def rate
        device = params[:device]
        locator = params[:locator]
        value = params[:rating]
        ride = Ride.for_device_and_locator(device, locator)
        ride.rate!(value)
        head :no_content
      end

      protected

      # comprueba la disponibilidad antes de permitir hacer una solicitud
      def check_availability
        available_taxis = Taxi.where(available: true)
        if available_taxis.empty?
          render json: { error: true, message: NO_AVAILABILITY }
        end
      end
    end
  end
end
