# encoding : utf-8
module Api
  module V1
    class TaxiController < ApplicationController
      respond_to :json

      # antes de cada accion hay que comprobar que el taxista este autenticado
      before_filter :authenticate_taxi_driver!

      # actualizar posicion del taxi perteneciente al taxista
      def update_position
        update_taxi_with_attributes(latitude: params[:latitude], longitude: params[:longitude])
      end

      # obtener informacion del taxi perteneciente al taxista
      def taxi_information
        @taxi = current_taxi_driver.taxi
      end

      # actualizar disponibilidad del taxi perteneciente al taxista
      def update_availability
        update_taxi_with_attributes(available: params[:available])
      end

      # actualizar identificador del dispositivo del taxista
      def update_device_id
        if current_taxi_driver.update_attributes(device: params[:device])
          head :no_content
        else
          render json: current_taxi_driver.errors, status: :unprocessable_entity
        end
      end

      # aceptar solicitud actual
      def accept_ride
        assignment = current_taxi_driver.taxi.current_assignment
        assignment.accept_assignment
        head :no_content
      end

      # rechazar solicitud actual
      def reject_ride
        assignment = current_taxi_driver.taxi.current_assignment
        assignment.reject_assignment
        head :no_content
      end

      # obtener informacion del servicio actual
      def ride_information
        @ride = current_taxi_driver.taxi.current_assignment.ride
      end

      # obtener informacion de la solicitud actual
      def assignment_information
        @assignment = current_taxi_driver.taxi.current_assignment
      end

      # iniciar servicio para la solicitud actual (al llegar al cliente)
      def begin_ride
        ride = current_taxi_driver.taxi.current_assignment.ride
        ride.begin!
        head :no_content
      end

      # finalizar servicio actual
      def end_ride
        ride = current_taxi_driver.taxi.current_assignment.ride
        ride.end!
        head :no_content
      end

      # obtener reputacion del taxista
      def reputation
        @ratings = current_taxi_driver.taxi.ratings
      end

      protected

      # metodo comun para actualizar atributos del taxi perteneciente al taxista
      def update_taxi_with_attributes(attributes = {})
        if current_taxi_driver.taxi.update_attributes(attributes)
          head :no_content
        else
          render json: current_taxi_driver.taxi.errors, status: :unprocessable_entity
        end
      end

    end
  end
end
