# encoding : utf-8
class Notification

  # identificadores de notificaciones
  NOTIF_NEW_RIDE = "NEW_RIDE"
  NOTIF_ACCEPTED = "ACCEPTED"
  NOTIF_ARRIVAL = "ARRIVAL"
  NOTIF_RATE = "RATE"

  # ficheros con certificados para notificaciones push
  CLIENT_CERT  = "config/push/client.pem"
  DRIVER_CERT  = "config/push/driver.pem"
  CLIENT_CERT_DEV  = "config/push/client-dev.pem"
  DRIVER_CERT_DEV  = "config/push/driver-dev.pem"

  # para enviar notificaciones al dispositivo deseado
  def self.send(token, message, to)
    # prepara notificacion para el dispositivo con el mensaje indicado
    notification = Grocer::Notification.new(
      device_token: token.gsub(/[<> ]/, ""),
      sound:        "default",
      badge:        1,
      alert:        {
        "loc-key" => message
      }
    )

    # selecciona el certificado adecuado segun el destinatario y el entorno de ejecucion
    cert = (to == :client) ? CLIENT_CERT_DEV : DRIVER_CERT_DEV
    cert = (to == :client) ? CLIENT_CERT : DRIVER_CERT if Rails.env.production?

    connection_options = {
      certificate: Rails.root.join(cert)
    }

    # envia notificacion
    pusher = Grocer.pusher(connection_options)
    pusher.push(notification)
  end

end