# encoding : utf-8
class Taxi < ActiveRecord::Base
  attr_accessible :identifier, :available, :latitude, :longitude

  # relaciones con otros modelos
  belongs_to :taxi_driver
  has_many :ratings
  belongs_to :current_assignment, class_name: "Assignment"

  # filtros usados habitualmente
  scope :available, where(available: true)
  scope :unassigned, where(current_assignment_id: nil)

  # calcula distancia euclidea del taxi a la posicion indicada
  def distance_to(lat, long)
    Math.sqrt([self.latitude, self.longitude].zip([lat, long]).map{|a,b| a-b}.map{|d| d*d}.reduce(:+))
  end

  # obtiene lista de taxis mas cercanos a la posicion indicada, ordenados por distancia
  def self.closest_taxis(lat, long)
    if lat.present? and long.present?
      taxis = available.map do |taxi|
        {
          taxi: taxi,
          distance: taxi.distance_to(lat, long)
        }
      end
      ordered_taxis = taxis.sort_by { |x| x[:distance] }
      ordered_taxis
    end
  end
end
