DEVICE_ID = "DEVICE_ID"
DEVICE_LAT = 39.494 # son ferrer
DEVICE_LONG = 2.5
RATING = 4

Given(/^there are taxis$/) do
  taxi_drivers = FactoryGirl.create_list(:taxi_driver, 6)
  @taxis = [
    FactoryGirl.create(:taxi, identifier: "1", available: true , latitude: 39.5170, longitude: 2.4830, taxi_driver: taxi_drivers[0]), # sta ponsa
    FactoryGirl.create(:taxi, identifier: "2", available: true , latitude: 39.5744, longitude: 2.4205, taxi_driver: taxi_drivers[1]), # andratx
    FactoryGirl.create(:taxi, identifier: "3", available: true , latitude: 39.5655, longitude: 2.5060, taxi_driver: taxi_drivers[2]), # calvia
    FactoryGirl.create(:taxi, identifier: "4", available: false, latitude: 39.5064, longitude: 2.5290, taxi_driver: taxi_drivers[3]), # magaluf
    FactoryGirl.create(:taxi, identifier: "5", available: false, latitude: 39.5011, longitude: 2.4680, taxi_driver: taxi_drivers[4]), # sta ponsa
    FactoryGirl.create(:taxi, identifier: "6", available: false, latitude: 39.5377, longitude: 2.5710, taxi_driver: taxi_drivers[5])  # portals
  ]
end

Given(/^there are available taxis$/) do
  step "there are taxis"
  @taxis.each { |taxi| taxi.available = true }.map(&:save)
end

Given(/^there are no available taxis$/) do
  step "there are taxis"
  @taxis.each { |taxi| taxi.available = false }.map(&:save)
end

Given(/^there is a new hailed ride$/) do
  @hailed_ride = Ride.create(device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG)
end

Given(/^there is a scheduled pickup coming up$/) do
  @time = 1.hours.from_now
  @pickup = Ride.create(device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG, booking_time: @time)
end

Given(/^I have an assigned taxi$/) do
  step 'there are available taxis'
  @hailed_ride = Ride.create(device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG)
  @hailed_ride.assign_taxi
  @hailed_ride.current_assignment.accept_assignment
end

Given(/^I hailed a taxi$/) do
  step "I have an assigned taxi"
end

Given(/^the ride is complete$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.begin!
  @hailed_ride.end!
end

Given(/^I have requested a taxi$/) do
  step 'there are available taxis'
  @hailed_ride = Ride.create(device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG)
end

When(/^I request the list of available taxis$/) do
  header 'Accept', 'application/json'
  get '/api/v1/available_taxis'
end

When(/^I hail a taxi$/) do
  header 'Accept', 'application/json'
  post '/api/v1/hail_taxi', { device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG }
end

When(/^I schedule a pickup for later$/) do
  @time = 3.hours.from_now
  header 'Accept', 'application/json'
  post '/api/v1/schedule_pickup', { device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG, booking_time: @time }
end

When(/^the system assigns a taxi to the pending ride$/) do
  @hailed_ride.assign_taxi
end

When(/^the system assigns a taxi to the scheduled pickup$/) do
  @pickup.assign_taxi
end

When(/^I request the assigned taxi information$/) do
  header 'Accept', 'application/json'
  get '/api/v1/assigned_taxi', { device: DEVICE_ID, locator: @hailed_ride.locator }
end

When(/^I request the requested ride information$/) do
  header 'Accept', 'application/json'
  get '/api/v1/requested_ride', { device: DEVICE_ID, locator: @hailed_ride.locator }
end

When(/^I request the latest unrated ride$/) do
  header 'Accept', 'application/json'
  get '/api/v1/unrated', { device: DEVICE_ID }
end

When(/^I rate the last ride$/) do
  header 'Accept', 'application/json'
  post '/api/v1/rate', { device: DEVICE_ID, locator: @hailed_ride.locator, rating: RATING }
end

Then(/^I should only see the available taxis and their positions$/) do
  available_taxis_json = JSON last_response.body
  expected_available_taxis = JSON.parse(%{
    [
      {
        "identifier": "1",
        "available": true,
        "latitude": 39.5170,
        "longitude": 2.4830
      },
      {
        "identifier": "2",
        "available": true,
        "latitude": 39.5744,
        "longitude": 2.4205
      },
      {
        "identifier": "3",
        "available": true,
        "latitude": 39.5655,
        "longitude": 2.5060
      }
    ]
  })
  available_taxis_json.should == expected_available_taxis
end

Then(/^I should have a booking$/) do
  hailing_json = JSON last_response.body
  hailing_json.should have_key("locator")
  hailing_json["locator"].should be_present
  hailing_json.should have_key("created_at")
  hailing_json["created_at"].should be_present
  hailing_json.should have_key("booking_time")
  hailing_json["booking_time"].should be_present
  hailing_json.should have_key("latitude")
  hailing_json["latitude"].should == DEVICE_LAT
  hailing_json.should have_key("longitude")
  hailing_json["longitude"].should == DEVICE_LONG
  hailing_json.should have_key("device")
  hailing_json["device"].should == DEVICE_ID
end

Then(/^I should receive an error message with code "(.*?)"$/) do |message|
  message_json = JSON last_response.body
  message_json.should have_key("error")
  message_json["error"].should == true
  message_json.should have_key("message")
  message_json["message"].should == message
end

Then(/^the ride should have been assigned the closest taxi$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "assigned"
  @hailed_ride.current_assignment.taxi.identifier.should == "1" # el mas cercano a son ferrer es santa ponsa
end

Then(/^the pickup should have been assigned the closest taxi$/) do
  @pickup = Ride.find @pickup.id
  @pickup.status.should == "assigned"
  @pickup.current_assignment.taxi.identifier.should == "1" # el mas cercano a son ferrer es santa ponsa
end

Then(/^the taxi driver should have been notified of the ride$/) do
  @hailed_ride.current_assignment.notified_at.should be_present
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_NEW_RIDE
end

Then(/^the taxi driver should have been notified of the pickup$/) do
  @pickup.current_assignment.notified_at.should be_present
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_NEW_RIDE
end

Then(/^I should see its position$/) do
  assigned_taxi = JSON last_response.body
  assigned_taxi.should have_key("identifier")
  assigned_taxi["identifier"].should be_present
  assigned_taxi.should have_key("latitude")
  assigned_taxi["latitude"].should be_present
  assigned_taxi.should have_key("longitude")
  assigned_taxi["longitude"].should be_present
end

Then(/^the driver will have a new rating$/) do
  @hailed_ride.current_assignment.taxi.ratings.should_not be_empty
end

Then(/^its status should be (.*?)$/) do |status|
  requested_ride = JSON last_response.body
  requested_ride.should have_key("locator")
  requested_ride["locator"].should be_present
  requested_ride.should have_key("status")
  requested_ride["status"].should == status
end
