TAXI_DEVICE_LAT = 39.49 # el toro
TAXI_DEVICE_LONG = 2.48
TAXI_DEVICE_ID = "TAXI_DEVICE_ID"

Given(/^I am logged in as a taxi driver$/) do
  post '/api/v1/tokens', { username: @driver.username, password: @driver.password }
  @token = JSON(last_response.body)["token"]
end

Given(/^I am logged out as a taxi driver$/) do
  step 'I am logged in as a taxi driver'
  delete "/api/v1/tokens/#{@token}"
  last_response.status.should == 200
end

Given(/^I am a taxi driver$/) do
  @driver = FactoryGirl.create(:taxi_driver) do |taxi_driver|
    # para asegurarnos que no coincide
    taxi_driver.taxi = FactoryGirl.create(:taxi, latitude: TAXI_DEVICE_LAT + 0.5, longitude: TAXI_DEVICE_LONG + 0.5)
  end
end

Given(/^I am an authenticated taxi driver$/) do
  step 'I am a taxi driver'
  step 'I am logged in as a taxi driver'
end

Given(/^there are other available taxis$/) do
  taxi_drivers = FactoryGirl.create_list(:taxi_driver, 6)
  1.upto(6) do |n|
    FactoryGirl.create(:taxi, latitude: TAXI_DEVICE_LAT + 0.5 + (0.01*n), longitude: TAXI_DEVICE_LONG + 0.5 + (0.01*n), taxi_driver: taxi_drivers[n])
  end
end

Given(/^I am an available taxi driver$/) do
  step 'I am a taxi driver'
  step 'I am logged in as a taxi driver'
  @driver.taxi.update_attributes(available: true)
end

Given(/^I am an unavailable taxi driver$/) do
  step 'I am a taxi driver'
  step 'I am logged in as a taxi driver'
  @driver.taxi.update_attributes(available: false)
end

Given(/^I updated my device identifier$/) do
  header 'Accept', 'application/json'
  post '/api/v1/update_device_id', { auth_token: @token, device: TAXI_DEVICE_ID }
  last_response.status.should_not == 401
end

Given(/^the system has assigned me a ride$/) do
  @hailed_ride = Ride.create(device: DEVICE_ID, latitude: TAXI_DEVICE_LAT, longitude: TAXI_DEVICE_LONG)
  @hailed_ride.assignments.create(taxi: @driver.taxi)
end

Given(/^the system has notified me$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "assigned"
  @hailed_ride.current_assignment.notified_at.should be_present
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_NEW_RIDE
end

Given(/^I have an accepted ride$/) do
  step "I am an available taxi driver"
  step "there are other available taxis"
  step "I updated my device identifier"
  step "the system has assigned me a ride"
  step "the system has notified me"
  step "I accept the ride"
  step "the customer should be notified"
end

Given(/^I started the ride$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.begin!
  step "the customer should be notified of my arrival"
end

When(/^a client requests a ride$/) do
  header 'Accept', 'application/json'
  post '/api/v1/hail_taxi', { device: DEVICE_ID, latitude: TAXI_DEVICE_LAT, longitude: TAXI_DEVICE_LONG }
  locator = JSON(last_response.body)["locator"]
  @hailed_ride = Ride.find_by_locator locator
end

When(/^I update my position$/) do
  header 'Accept', 'application/json'
  post '/api/v1/update_position', { auth_token: @token, latitude: TAXI_DEVICE_LAT, longitude: TAXI_DEVICE_LONG }
  last_response.status.should_not == 401
end

When(/^I get my taxi's information$/) do
  header 'Accept', 'application/json'
  get '/api/v1/taxi_information', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I set my status to available$/) do
  header 'Accept', 'application/json'
  post '/api/v1/update_availability', { auth_token: @token, available: true }
  last_response.status.should_not == 401
end

When(/^I set my status to not available$/) do
  header 'Accept', 'application/json'
  post '/api/v1/update_availability', { auth_token: @token, available: false }
  last_response.status.should_not == 401
end

When(/^I accept the ride$/) do
  header 'Accept', 'application/json'
  post '/api/v1/accept_ride', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I reject the ride$/) do
  header 'Accept', 'application/json'
  post '/api/v1/reject_ride', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I request the ride information$/) do
  header 'Accept', 'application/json'
  get '/api/v1/ride_information', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I login as a taxi driver$/) do
  step "I am logged in as a taxi driver"
end

When(/^I logout as a taxi driver$/) do
  @driver = TaxiDriver.find @driver.id
  delete "/api/v1/tokens/#{@driver.authentication_token}"
  last_response.status.should == 200
end

When(/^I get close to the customer$/) do
  header 'Accept', 'application/json'
  post '/api/v1/begin_ride', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I finish the ride$/) do
  header 'Accept', 'application/json'
  post '/api/v1/end_ride', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I request my reputation$/) do
  header 'Accept', 'application/json'
  get '/api/v1/reputation', { auth_token: @token }
  last_response.status.should_not == 401
end

When(/^I login with invalid taxi driver credentials$/) do
  post '/api/v1/tokens', { username: 'invalid', password: 'invalid' }
end

When(/^I request my current assignment$/) do
  header 'Accept', 'application/json'
  get '/api/v1/assignment_information', { auth_token: @token }
  last_response.status.should_not == 401
end

Then(/^my taxi should have my current position$/) do
  @driver = TaxiDriver.find @driver.id
  @driver.taxi.latitude.should == TAXI_DEVICE_LAT
  @driver.taxi.longitude.should == TAXI_DEVICE_LONG
end

Then(/^I should see my taxi's availability and position$/) do
  taxi = JSON last_response.body
  taxi.should have_key("identifier")
  taxi["identifier"].should be_present
  taxi.should have_key("latitude")
  taxi["latitude"].should be_present
  taxi.should have_key("longitude")
  taxi["longitude"].should be_present
  taxi.should have_key("available")
  taxi["available"].should be_present
end

Then(/^my taxi should be available$/) do
  @driver = TaxiDriver.find @driver.id
  @driver.taxi.available.should == true
end

Then(/^my taxi should not be available$/) do
  @driver = TaxiDriver.find @driver.id
  @driver.taxi.available.should == false
end

Then(/^the ride should be accepted$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "accepted"
  @hailed_ride.accepted_assignment.status.should == "accepted"
end

Then(/^the customer should be notified$/) do
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_ACCEPTED
end

Then(/^the ride should be rejected$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "assigned"
  @hailed_ride.assignments.for_driver(@driver).first.status.should == "rejected"
end

Then(/^the system should have assigned another taxi$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "assigned"
  @hailed_ride.current_assignment.taxi.id.should_not == @driver.taxi.id
end

Then(/^the newly assigned taxi driver should have been notified$/) do
  @hailed_ride.current_assignment.notified_at.should be_present
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_NEW_RIDE
end

Then(/^I should view the customer's position$/) do
  current_ride = JSON last_response.body
  current_ride.should have_key("locator")
  current_ride["locator"].should be_present
  current_ride.should have_key("latitude")
  current_ride["latitude"].should be_present
  current_ride.should have_key("longitude")
  current_ride["longitude"].should be_present
  current_ride.should have_key("status")
  current_ride["status"].should be_present
end

Then(/^I should see my current assignment$/) do
  current_assignment = JSON last_response.body
  current_assignment.should have_key("status")
  current_assignment["status"].should be_present
  current_assignment.should have_key("ride")
  current_assignment["ride"].should have_key("locator")
  current_assignment["ride"]["locator"].should be_present
  current_assignment["ride"].should have_key("latitude")
  current_assignment["ride"]["latitude"].should be_present
  current_assignment["ride"].should have_key("longitude")
  current_assignment["ride"]["longitude"].should be_present
  current_assignment["ride"].should have_key("status")
  current_assignment["ride"]["status"].should be_present
end

Then(/^I should be logged in as a taxi driver$/) do
  @driver = TaxiDriver.find @driver.id
  @driver.authentication_token.should be_present
end

Then(/^I should be logged out as a taxi driver$/) do
  @driver = TaxiDriver.find @driver.id
  @driver.authentication_token.should_not == @token
end

Then(/^the ride should have begun$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "en_route"
end

Then(/^the customer should be notified of my arrival$/) do
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_ARRIVAL
end

Then(/^the ride should be complete$/) do
  @hailed_ride = Ride.find @hailed_ride.id
  @hailed_ride.status.should == "complete"
end

Then(/^the customer should be notified in order to rate the ride$/) do
  notification = @server.notifications.pop
  notification.alert["loc-key".to_sym].should == Notification::NOTIF_RATE
end

Then(/^I should view my ratings$/) do
  ratings = JSON last_response.body
  ratings.should have_key("average")
  ratings["average"].should be_present
  ratings.should have_key("count")
  ratings["count"].should be_present
  ratings.should have_key("ratings")
end

Then(/^I should receive an invalid credentials error message$/) do
  last_response.status.should == 401
  login_response = JSON(last_response.body)
  login_response.should have_key("message")
end
